Ralpb = LibStub("AceAddon-3.0"):NewAddon("RalpbQuest", "AceConsole-3.0", "AceEvent-3.0")

-- Local vars
local CURRENT_VERSION = GetAddOnMetadata("RalpbQuest", "Version")
local RQ_CHAT_TYPE = "PARTY"
local RQ_QUEST_TEXT = "(.*): %s*([-%d]+)%s*/%s*([-%d]+)%s*$";
local RQ_QUEST_INCOMPLETE = {}

-- Start up 
function Ralpb:OnEnable()
	self:RegisterSlashCommands()

	self:RegisterEvent("QUEST_LOG_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("UI_INFO_MESSAGE")

	if self:InParty() then
		RQ_CHAT_TYPE = "PARTY"
	else
		RQ_CHAT_TYPE = "SAY"
	end

	self:InitQuestArray()
	--self:CheckQuestComplete()
end

function Ralpb:InitQuestArray()
	local numEntries, numQuests = GetNumQuestLogEntries()

	for i = 1, numEntries do
		SelectQuestLogEntry(i)
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(i)

		if not isHeader then
			--self:Print(i .. " : " .. questTitle .. " : " .. level)

			if RQ_QUEST_INCOMPLETE[self:GetQID()] == nil then

				--self:Print(self:GetQID() .. " TOEGEVOEGD!")
			
				if isComplete == nil then
					RQ_QUEST_INCOMPLETE[self:GetQID()] = false
				elseif isComplete == -1 then
					RQ_QUEST_INCOMPLETE[self:GetQID()] = false
				else
					RQ_QUEST_INCOMPLETE[self:GetQID()] = true
				end
			end
		end
	end

--	for index, value in pairs(RQ_QUEST_INCOMPLETE) do
--		self:Print(index.." : ".. tostring(value))
--	end
end

function Ralpb:CheckQuestComplete()

	local numEntries, numQuests = GetNumQuestLogEntries()

	for i = 1, numEntries do
		SelectQuestLogEntry(i)
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily = GetQuestLogTitle(i)
		
		if not isHeader then
			local questCompleted = true

			if isComplete == nil or isComplete == -1 then
				questCompleted = false
			end

			--self:Print("questTitle: " .. questTitle .. " COMPLETED: " .. tostring(questCompleted))
	
			if RQ_QUEST_INCOMPLETE[self:GetQID()] == false then
				if questCompleted == true  then
					-- Quest is completed
					PlaySoundFile("sound/interface/igplayerBind.wav")
					self:SendNotification("[" .. questTitle .."] " .. "is now completed!")

					RQ_QUEST_INCOMPLETE[self:GetQID()] = true
				end
			end
		end
	end
end

function Ralpb:GetQID()
	local qindex = GetQuestLogSelection()
	local link = GetQuestLink(qindex)
	return tonumber(link:match(":(%d+):"))	
end

-- Events
function Ralpb:QUEST_LOG_UPDATE()
	self:InitQuestArray()
	self:CheckQuestComplete()
end

function Ralpb:PARTY_MEMBERS_CHANGED()
	if self:InParty() then
		RQ_CHAT_TYPE = "PARTY"
		--self:Print("Output in " .. RQ_CHAT_TYPE)
	else
		RQ_CHAT_TYPE = "SAY"
		--self:Print("Output in " .. RQ_CHAT_TYPE)
	end
end

function Ralpb:UI_INFO_MESSAGE(arg1, arg2)
	local uQuestText = gsub(arg2, RQ_QUEST_TEXT, "%1", 1);

	if (uQuestText ~= arg2) then
		self:SendNotification(arg2)
	end
end

function Ralpb:InParty()
	local returnValue = false

	if GetNumPartyMembers() > 0 then
		returnValue = true
	end

	return returnValue
end

-- Slash commands
function Ralpb:RegisterSlashCommands()
	self:RegisterChatCommand('ralpb', 'OnCmd')
end

function Ralpb:OnCmd(cmd)
	if cmd == 'reloadui' then
		self:Reload()
	elseif cmd == 'version' then
		self:PrintVersion()
	elseif cmd == 'party' then
		if self:InParty() then
			RQ_CHAT_TYPE = "PARTY"
			self:Print("Output in " .. RQ_CHAT_TYPE)
		end
	elseif cmd == 'say' then
		RQ_CHAT_TYPE = "SAY"
		self:Print("Output in " .. RQ_CHAT_TYPE)
	else
		self:PrintInfo()
	end
end

function Ralpb:PrintVersion()
	self:Print("Version: " .. CURRENT_VERSION)
end

function Ralpb:Reload()
	ReloadUI()
end

function Ralpb:PrintInfo()
	self:Print("RalpbQuest is an addon made by Ralpbix on Daggerspine")
end

function Ralpb:SendNotification(message)
	SendChatMessage(message, RQ_CHAT_TYPE);
end